@@ -1,17 +1,17 @@
-from __future__ import unicode_literals
 
-import django.forms.models as model_forms
+
 import json
 import operator
-import six
+from functools import reduce
 
+import django.forms.models as model_forms
 from django import forms
 from django.conf import settings
 from django.conf.urls import url
 from django.contrib import messages
-from django.contrib.auth import REDIRECT_FIELD_NAME, get_user_model
+from django.contrib.auth import REDIRECT_FIELD_NAME
 from django.core.exceptions import ImproperlyConfigured
-from django.core.urlresolvers import reverse
+from django.urls import reverse
 from django.db import IntegrityError
 from django.db.models import Q
 from django.http import HttpResponseRedirect, HttpResponse, JsonResponse
@@ -21,11 +21,10 @@ from django.utils.translation import ugettext_lazy as _
 from django.views.generic.edit import ModelFormMixin, UpdateView, CreateView, ProcessFormView, FormView
 from django.views.generic.base import TemplateView
 from django.views.generic import DetailView, ListView
-from functools import reduce
-from guardian.shortcuts import get_objects_for_user, assign_perm
-from guardian.utils import get_anonymous_user
+
 from smartmin.csv_imports.models import ImportTask
 from smartmin.mixins import NonAtomicMixin
+
 from . import widgets
 
 
@@ -97,34 +96,7 @@ class SmartView(object):
         if not getattr(self, 'permission', None):
             return True
         else:
-            # first check our anonymous permissions
-            real_anon = get_anonymous_user()
-            has_perm = real_anon.has_perm(self.permission)
-
-            # if not, then check our real permissions
-            if not has_perm:
-                has_perm = request.user.has_perm(self.permission)
-
-            # if not, perhaps we have it per object
-            if not has_perm:
-                has_perm = self.has_object_permission('get_object')
-
-            # if still no luck, check if we have permissions on the parent object
-            if not has_perm:
-                has_perm = self.has_object_permission('get_parent_object')
-
-            return has_perm
-
-    def has_object_permission(self, getter_name):
-        """
-        Checks for object level permission for an arbitrary getter
-        """
-        obj_getter = getattr(self, getter_name, None)
-
-        if obj_getter:
-            obj = obj_getter()
-            if obj:
-                return self.request.user.has_perm(getattr(self, 'permission', None), obj)
+            return request.user.has_perm(self.permission)
 
     def dispatch(self, request, *args, **kwargs):
         """
@@ -351,7 +323,7 @@ class SmartView(object):
         url_params = "?"
         order_params = ""
         for key in self.request.GET.keys():
-            if key != 'page' and key != 'pjax' and key[0] != '_':
+            if key != 'page' and key != 'pjax' and (len(key) == 0 or key[0] != '_'):
                 for value in self.request.GET.getlist(key):
                     url_params += "%s=%s&" % (key, urlquote(value))
             elif key == '_order':
@@ -506,7 +478,7 @@ class SmartDeleteView(SmartSingleObjectView, DetailView, ProcessFormView):
 
     def pre_delete(self, obj):
         # auto populate modified_by if it is present
-        if hasattr(obj, 'modified_by_id') and self.request.user.id >= 0:
+        if self.request.user.id and self.request.user.id > 0 and hasattr(obj, 'modified_by_id'):
             obj.modified_by = self.request.user
 
     def post(self, request, *args, **kwargs):
@@ -543,8 +515,6 @@ class SmartListView(SmartView, ListView):
     default_order = None
     select_related = None
 
-    list_permission = None
-
     @classmethod
     def derive_url_pattern(cls, path, action):
         if action == 'list':
@@ -601,7 +571,7 @@ class SmartListView(SmartView, ListView):
         try:
             self.model._meta.get_field_by_name(field)
             return True
-        except:
+        except Exception:
             # that field doesn't exist, so not sortable
             return False
 
@@ -645,11 +615,10 @@ class SmartListView(SmartView, ListView):
 
         # apply any filtering
         search_fields = self.derive_search_fields()
-        search_terms = self.request.GET.get('search', '').split(' ')
-        if search_fields and search_terms:
-
+        search_query = self.request.GET.get('search')
+        if search_fields and search_query:
             term_queries = []
-            for term in search_terms:
+            for term in search_query.split(' '):
                 field_queries = []
                 for field in search_fields:
                     field_queries.append(Q(**{field: term}))
@@ -672,16 +641,6 @@ class SmartListView(SmartView, ListView):
         """
         queryset = self.derive_queryset(**kwargs)
 
-        # if our list should be filtered by a permission as well, do so
-        if self.list_permission:
-            # only filter if this user doesn't have a global permission
-            if not self.request.user.has_perm(self.list_permission):
-                user = self.request.user
-                # guardian only behaves with model users
-                if settings.ANONYMOUS_USER_ID and user.is_anonymous():
-                    user = get_user_model().objects.get(pk=settings.ANONYMOUS_USER_ID)
-                queryset = queryset.filter(id__in=get_objects_for_user(user, self.list_permission))
-
         return self.order_queryset(queryset)
 
     def derive_ordering(self):
@@ -712,7 +671,7 @@ class SmartListView(SmartView, ListView):
 
         if order:
             # if our order is a single string, convert to a simple list
-            if isinstance(order, six.string_types):
+            if isinstance(order, str):
                 order = (order,)
 
             queryset = queryset.order_by(*order)
@@ -763,7 +722,9 @@ class SmartListView(SmartView, ListView):
 
                 results.append(result)
 
-            json_data = dict(results=results, err='nil', more=context['page_obj'].has_next())
+            has_more = context['page_obj'].has_next() if context['page_obj'] else False
+
+            json_data = dict(results=results, err='nil', more=has_more)
             return JsonResponse(json_data)
         # otherwise, return normally
         else:
@@ -792,14 +753,14 @@ class SmartCsvView(SmartListView):
         # build up our header row
         header = []
         for field in fields:
-            header.append(six.text_type(self.lookup_field_label(dict(), field)))
+            header.append(str(self.lookup_field_label(dict(), field)))
         writer.writerow([s.encode("utf-8") for s in header])
 
         # then our actual values
         for obj in self.object_list:
             row = []
             for field in fields:
-                row.append(six.text_type(self.lookup_field_value(dict(), obj, field)))
+                row.append(str(self.lookup_field_value(dict(), obj, field)))
             writer.writerow([s.encode("utf-8") for s in row])
 
         return response
@@ -823,14 +784,14 @@ class SmartXlsView(SmartListView):
         # build up our header row
         for col in range(len(fields)):
             field = fields[col]
-            sheet1.write(0, col, six.text_type(self.lookup_field_label(dict(), field)))
+            sheet1.write(0, col, str(self.lookup_field_label(dict(), field)))
 
         # then our actual values
         for row in range(len(self.object_list)):
             obj = self.object_list[row]
             for col in range(len(fields)):
                 field = fields[col]
-                value = six.text_type(self.lookup_field_value(dict(), obj, field))
+                value = str(self.lookup_field_value(dict(), obj, field))
                 # skip the header
                 sheet1.write(row + 1, col, value)
 
@@ -912,6 +873,7 @@ class SmartFormMixin(object):
         """
         if isinstance(field, forms.fields.DateField) and isinstance(field.widget, forms.widgets.DateInput):
             field.widget = widgets.DatePickerWidget()
+            field.input_formats = [field.widget.input_format[1]] + list(field.input_formats)
 
         if isinstance(field, forms.fields.ImageField) and isinstance(field.widget, forms.widgets.ClearableFileInput):
             field.widget = widgets.ImageThumbnailWidget()
@@ -1093,7 +1055,6 @@ class SmartFormView(SmartFormMixin, SmartView, FormView):
 
 
 class SmartModelFormView(SmartFormMixin, SmartSingleObjectView, ModelFormMixin):
-    grant_permissions = None
     javascript_submit = None
 
     field_config = {'modified_blurb': dict(label="Modified"), 'created_blurb': dict(label="Created")}
@@ -1152,14 +1113,6 @@ class SmartModelFormView(SmartFormMixin, SmartSingleObjectView, ModelFormMixin):
         """
         Called after an object is successfully saved
         """
-        # if we have permissions to grant, do so
-        if self.grant_permissions:
-            for permission in self.grant_permissions:
-                # if the user doesn't have this permission globally already
-                if not self.request.user.has_perm(permission):
-                    # then assign it for this object
-                    assign_perm(permission, self.request.user, self.object)
-
         return obj
 
     def get_context_data(self, **kwargs):
@@ -1196,7 +1149,7 @@ class SmartUpdateView(SmartModelFormView, UpdateView):
 
     def pre_save(self, obj):
         # auto populate modified_by if it is present
-        if hasattr(obj, 'modified_by_id') and self.request.user.id >= 0:
+        if self.request.user.id and self.request.user.id > 0 and hasattr(obj, 'modified_by_id'):
             obj.modified_by = self.request.user
 
         return obj
@@ -1304,13 +1257,14 @@ class SmartCreateView(SmartModelFormView, CreateView):
         return False
 
     def pre_save(self, obj):
-        # auto populate created_by if it is present
-        if hasattr(obj, 'created_by_id') and self.request.user.id >= 0:
-            obj.created_by = self.request.user
+        if self.request.user.id and self.request.user.id > 0:
+            # auto populate created_by if it is present
+            if hasattr(obj, 'created_by_id'):
+                obj.created_by = self.request.user
 
-        # auto populate modified_by if it is present
-        if hasattr(obj, 'modified_by_id') and self.request.user.id >= 0:
-            obj.modified_by = self.request.user
+            # auto populate modified_by if it is present
+            if hasattr(obj, 'modified_by_id'):
+                obj.modified_by = self.request.user
 
         return obj
 
