@@ -1,30 +1,28 @@
-from __future__ import unicode_literals
-
 import json
 import pytz
-import smartmin
+from unittest.mock import patch
 
 from datetime import datetime, timedelta
 from django.conf import settings
 from django.contrib.auth.models import User, Group
 from django.core import mail
 from django.core.files.uploadedfile import SimpleUploadedFile
-from django.core.urlresolvers import reverse
+from django.urls import reverse
 from django.test import TestCase
 from django.test.client import Client
 from django.test.utils import override_settings
 from django.utils import timezone
-from guardian.conf import settings as guardian_settings
-from guardian.shortcuts import assign_perm
-from mock import patch
+
+import smartmin
 from smartmin.csv_imports.models import ImportTask
 from smartmin.management import check_role_permissions
 from smartmin.models import SmartImportRowError
 from smartmin.templatetags.smartmin import view_as_json, get_value_from_view, get, user_as_string
 from smartmin.tests import SmartminTest
-from smartmin.users.models import FailedLogin, RecoveryToken, PasswordHistory
+from smartmin.users.models import FailedLogin, RecoveryToken, PasswordHistory, is_password_complex
 from smartmin.views import smart_url
-from smartmin.widgets import ImageThumbnailWidget
+from smartmin.widgets import DatePickerWidget, ImageThumbnailWidget
+
 from test_runner.blog.models import Post, Category
 from .views import PostCRUDL
 
@@ -101,12 +99,12 @@ class PostTest(SmartminTest):
         # updating posts
         update_url = reverse('blog.post_update', args=[self.post.id])
 
-        # if not logged in can't read
+        # if not logged in can't update
         self.client.logout()
         response = self.client.get(update_url)
         self.assertIsLogin(response)
 
-        # plain user can't see it either
+        # plain user can't update it either
         self.assertNoAccess(self.plain, update_url)
 
         # but editors can, as can authors and superusers
@@ -114,31 +112,49 @@ class PostTest(SmartminTest):
         self.assertHasAccess(self.author, update_url)
         self.assertHasAccess(self.superuser, update_url)
 
-        # now test reading posts
+        # now test reading posts (doesn't have a set permission)
         read_url = reverse('blog.post_read', args=[self.post.id])
 
-        # if not logged in can still read
+        # even users not logged in can read
         self.client.logout()
         response = self.client.get(read_url)
         self.assertEquals(200, response.status_code)
 
-        # everybody else can read too
+        # along with everyone else
         self.assertHasAccess(self.plain, read_url)
         self.assertHasAccess(self.editor, read_url)
         self.assertHasAccess(self.author, read_url)
         self.assertHasAccess(self.superuser, read_url)
 
-        # now grant object level permission to update a single post for anonymous user
-        self.client.logout()
-        anon = User.objects.get(username=guardian_settings.ANONYMOUS_USER_NAME)
-        assign_perm('blog.post_update', anon, self.post)
-
-        response = self.client.get(update_url)
-        self.assertEquals(200, response.status_code)
-
-    def test_create(self):
+    def test_refresh_page(self):
         self.client.login(username='author', password='author')
 
+        refresh_url = reverse('blog.post_refresh', args=[self.post.id])
+
+        response = self.client.get(refresh_url)
+
+        self.assertContains(response, 'function scheduleRefresh')
+
+    def test_norefresh_page(self):
+        self.client.login(username='author', password='author')
+
+        no_refresh_url = reverse('blog.post_no_refresh', args=[self.post.id])
+
+        response = self.client.get(no_refresh_url)
+
+        self.assertNotContains(response, 'function scheduleRefresh')
+
+    def test_create_and_update(self):
+        self.client.login(username='author', password='author')
+
+        response = self.client.get(reverse('blog.post_create'))
+        self.assertEqual(response.status_code, 200)
+
+        date_field = response.context['form'].fields['written_on']
+        self.assertIsInstance(date_field.widget, DatePickerWidget)
+        self.assertEqual(date_field.widget.format, "%B %d, %Y")
+        self.assertEqual(date_field.input_formats, ['%B %d, %Y', '%Y-%m-%d', '%m/%d/%Y', '%m/%d/%y'])
+
         post_data = dict(title="New Post", body="This is a new post", order=1, tags="post")
         self.client.post(reverse('blog.post_create'), post_data, follow=True)
 
@@ -244,6 +260,75 @@ class PostTest(SmartminTest):
                 {'id': self.post.id, 'text': 'Test Post'}
             ], 'err': 'nil', 'more': False})
 
+        # check parsing of query string params used for paging URLs
+        response = self.client.get(reverse('blog.post_list') + "?=x&foo=bar&_order=-title&page=1")
+        self.assertEqual(list(response.context['post_list']), [self.post, post3, post2, post4, post1])
+        self.assertEqual(response.context['url_params'], '?=x&foo=bar&')
+        self.assertEqual(response.context['order_params'], '_order=-title&')
+
+    def test_list_no_pagination(self):
+        post1 = Post.objects.create(title="A First Post", body="Apples", order=3, tags="post",
+                                    created_by=self.author, modified_by=self.author)
+        post2 = Post.objects.create(title="A Second Post", body="Oranges", order=5, tags="post",
+                                    created_by=self.superuser, modified_by=self.superuser)
+        post3 = Post.objects.create(title="A Third Post", body="Apples", order=1, tags="post",
+                                    created_by=self.author, modified_by=self.author)
+        post4 = Post.objects.create(title="A Fourth Post", body="Oranges", order=3, tags="post",
+                                    created_by=self.superuser, modified_by=self.superuser)
+
+        self.client.login(username='author', password='author')
+
+        # default ordering is by title
+        response = self.client.get(reverse('blog.post_list_no_pagination'))
+        self.assertEqual(list(response.context['post_list']), [post1, post4, post2, post3, self.post])
+
+        # try ordering by title reversed
+        response = self.client.get(reverse('blog.post_list_no_pagination') + "?_order=-title")
+        self.assertEqual(list(response.context['post_list']), [self.post, post3, post2, post4, post1])
+
+        # try searching (which is case-insensitive)
+        response = self.client.get(reverse('blog.post_list_no_pagination') + "?search=apples")
+        self.assertEqual(list(response.context['post_list']), [post1, post3])
+
+        # multiple terms are AND'ed
+        response = self.client.get(reverse('blog.post_list_no_pagination') + "?search=apples+first")
+        self.assertEqual(list(response.context['post_list']), [post1])
+
+        # matches on different fields are OR'd
+        response = self.client.get(reverse('blog.post_list_no_pagination') + "?search=post")
+        self.assertEqual(list(response.context['post_list']), [post1, post4, post2, post3, self.post])
+
+        # empty search string should be ignored
+        response = self.client.get(reverse('blog.post_list_no_pagination') + "?search=")
+        self.assertEqual(list(response.context['post_list']), [post1, post4, post2, post3, self.post])
+
+        # change the format to json
+        response = self.client.get(reverse('blog.post_list_no_pagination') + "?_format=json")
+        self.assertEqual(json.loads(response.content.decode("utf-8")), [
+            {'tags': 'post', 'title': 'A First Post', 'body': 'Apples'},
+            {'tags': 'post', 'title': 'A Fourth Post', 'body': 'Oranges'},
+            {'tags': 'post', 'title': 'A Second Post', 'body': 'Oranges'},
+            {'tags': 'post', 'title': 'A Third Post', 'body': 'Apples'},
+            {'tags': 'testing_tag', 'title': 'Test Post', 'body': 'This is the body of my first test post'}
+         ])
+
+        # change the format to select2
+        response = self.client.get(reverse('blog.post_list_no_pagination') + "?_format=select2")
+        self.assertEqual(json.loads(response.content.decode("utf-8")), {
+            'results': [
+                {'id': post1.id, 'text': 'A First Post'},
+                {'id': post4.id, 'text': 'A Fourth Post'},
+                {'id': post2.id, 'text': 'A Second Post'},
+                {'id': post3.id, 'text': 'A Third Post'},
+                {'id': self.post.id, 'text': 'Test Post'}
+            ], 'err': 'nil', 'more': False})
+
+        # check parsing of query string params used for paging URLs
+        response = self.client.get(reverse('blog.post_list_no_pagination') + "?=x&foo=bar&_order=-title&page=1")
+        self.assertEqual(list(response.context['post_list']), [self.post, post3, post2, post4, post1])
+        self.assertEqual(response.context['url_params'], '?=x&foo=bar&')
+        self.assertEqual(response.context['order_params'], '_order=-title&')
+
     def test_success_url(self):
         self.client.login(username='author', password='author')
 
@@ -312,13 +397,13 @@ class PostTest(SmartminTest):
         permissions = ('blog.post.*', 'blog.post.too.many.dots', 'blog.category.not_valid_either', 'blog.',
                        'blog.foo.*')
 
-        self.assertEquals(17, authors.permissions.all().count())
+        self.assertEquals(18, authors.permissions.all().count())
 
         # check that they are reassigned
         check_role_permissions(authors, permissions, authors.permissions.all())
 
         # removing all category actions should bring us to 10
-        self.assertEquals(12, authors.permissions.all().count())
+        self.assertEquals(13, authors.permissions.all().count())
 
     def test_smart_model(self):
         d1 = datetime(2016, 12, 31, 9, 20, 30, 123456, tzinfo=pytz.timezone("Africa/Kigali"))
@@ -378,6 +463,9 @@ class PostTest(SmartminTest):
         with open('test_runner/blog/test_files/posts.csv', 'rb') as open_file:
             self.assertEqual(Post.get_import_file_headers(open_file), ['title', 'body', 'order', 'tags'])
 
+        with open('test_runner/blog/test_files/bom_import.csv', 'rb') as open_file:
+            self.assertEqual(Post.get_import_file_headers(open_file), ['urn:tel', 'name', 'field:email-address'])
+
     def test_csv_import(self):
         with self.settings(CELERY_ALWAYS_EAGER=True, CELERY_RESULT_BACKEND='cache', CELERY_CACHE_BACKEND='memory'):
             import_url = reverse('blog.post_csv_import')
@@ -400,6 +488,9 @@ class PostTest(SmartminTest):
             task = ImportTask.objects.get()
             self.assertEqual(json.loads(task.import_results), dict(records=4, errors=0, error_messages=[]))
 
+            # new posts should all have a new tag
+            self.assertEqual(Post.objects.filter(tags="new").count(), 4)
+
             ImportTask.objects.all().delete()
 
             csv_file = open('test_runner/blog/test_files/posts.xls', 'rb')
@@ -451,6 +542,15 @@ class UserTest(TestCase):
         self.superuser.is_superuser = True
         self.superuser.save()
 
+    def test_login_stripped_spaces(self):
+        login_url = reverse('users.user_login')
+
+        response = self.client.post(login_url, dict(username=' superuser', password='superuser'))
+        self.assertEquals(response.status_code, 302)
+
+        response = self.client.post(login_url, dict(username='    superuser     ', password='superuser'))
+        self.assertEquals(response.status_code, 302)
+
     def test_login_case_not_sensitive(self):
         login_url = reverse('users.user_login')
 
@@ -509,8 +609,8 @@ class UserTest(TestCase):
         self.assertEquals(200, response.status_code)
         self.assertTrue('new_password' in response.context['form'].errors)
 
-        # ok, finally with a zero in there too, this one should pass
-        post_data['new_password'] = 'Passw0rd'
+        # ok, finally with a zero and a whitespace in there too, this one should pass
+        post_data['new_password'] = 'Passw0rd '
         response = self.client.post(reverse('users.user_create'), post_data, follow=True)
         self.assertEquals(200, response.status_code)
         self.assertTrue('form' not in response.context)
@@ -518,6 +618,12 @@ class UserTest(TestCase):
         # make sure the user was created
         steve = User.objects.get(username='steve')
 
+        # test if we can login with steve
+        self.assertTrue(self.client.login(username='steve', password='Passw0rd '))
+
+        # login superuser again
+        self.client.login(username='superuser', password='superuser')
+
         # create another user manually, make him inactive
         woz = User.objects.create_user('woz', 'woz@apple.com', 'woz')
         woz.is_active = False
@@ -538,7 +644,7 @@ class UserTest(TestCase):
 
         # update steve, put him in a different group and change his password
         post_data['groups'] = Group.objects.get(name='Editors').id
-        post_data['new_password'] = 'googleIsNumber1'
+        post_data['new_password'] = ' googleIsNumber1'
 
         # need is active here or steve will be marked inactive
         post_data['is_active'] = '1'
@@ -555,7 +661,7 @@ class UserTest(TestCase):
         self.assertEquals(Group.objects.get(name='Editors'), groups[0])
 
         # assert steve can login with 'google' now
-        self.assertTrue(self.client.login(username='steve', password='googleIsNumber1'))
+        self.assertTrue(self.client.login(username='steve', password=' googleIsNumber1'))
 
         # test user profile action
         # logout first
@@ -594,7 +700,7 @@ class UserTest(TestCase):
         self.assertEquals(302, response.status_code)
 
         # log in as an editor instead
-        self.assertTrue(self.client.login(username='steve', password='googleIsNumber1'))
+        self.assertTrue(self.client.login(username='steve', password=' googleIsNumber1'))
         response = self.client.get(reverse('users.user_profile', args=[steve.id]))
         self.assertEquals(reverse('users.user_profile', args=[steve.id]), response.request['PATH_INFO'])
 
@@ -616,20 +722,21 @@ class UserTest(TestCase):
 
         # check if he can mismatch new password and its confirmation
         post_data = dict(old_password="plain", new_password="NewPassword1", confirm_new_password="confirmnewpassword")
-        self.client.post(reverse('users.user_profile', args=[steve.id]), post_data)
+        response = self.client.post(reverse('users.user_profile', args=[steve.id]), post_data)
+        self.assertTrue('confirm_new_password' in response.context['form'].errors)
 
         # check if he can fill old and new password only without the confirm new password
         post_data = dict(old_password="plain", new_password="NewPassword1")
         response = self.client.post(reverse('users.user_profile', args=[steve.id]), post_data)
         self.assertIn("Confirm the new password by filling the this field", response.content.decode("utf-8"))
 
-        # actually change the password
-        post_data = dict(old_password="googleIsNumber1", new_password="NewPassword1",
-                         confirm_new_password="NewPassword1")
-        self.client.post(reverse('users.user_profile', args=[steve.id]), post_data)
+        # actually change the password, with spaces
+        post_data = dict(old_password=" googleIsNumber1", new_password="NewPassword1 ",
+                         confirm_new_password="NewPassword1 ")
+        response = self.client.post(reverse('users.user_profile', args=[steve.id]), post_data)
 
         # assert new password works
-        self.assertTrue(self.client.login(username='steve', password='NewPassword1'))
+        self.assertTrue(self.client.login(username='steve', password='NewPassword1 '))
 
         # see whether we can change our email without a password
         post_data = dict(email="new@foo.com")
@@ -637,7 +744,7 @@ class UserTest(TestCase):
         self.assertTrue('old_password' in response.context['form'].errors)
 
         # but with the new password we can
-        post_data = dict(email="new@foo.com", old_password='NewPassword1')
+        post_data = dict(email="new@foo.com", old_password='NewPassword1 ')
         self.client.post(reverse('users.user_profile', args=[steve.id]), post_data)
         self.assertTrue(User.objects.get(email='new@foo.com'))
 
@@ -881,7 +988,7 @@ class UserTest(TestCase):
         # try to log in four times
         for i in range(4):
             response = self.client.post(login_url, post_data)
-            self.assertFalse(response.context['user'].is_authenticated())
+            self.assertFalse(response.context['user'].is_authenticated)
 
         # on the fifth failed login we get redirected
         response = self.client.post(login_url, post_data)
@@ -1027,14 +1134,14 @@ class UserLockoutTestCase(TestCase):
 
         # on the fifth time it should fail
         response = self.client.post(reverse('users.user_login'), post_data, follow=True)
-        self.assertFalse(response.context['user'].is_authenticated())
+        self.assertFalse(response.context['user'].is_authenticated)
         content = response.content.decode("utf-8")
         self.assertEqual(content.find(reverse('users.user_forget')), -1)
 
         # even with right password, no dice
         post_data = dict(username='plain', password='plain')
         response = self.client.post(reverse('users.user_login'), post_data, follow=True)
-        self.assertFalse(response.context['user'].is_authenticated())
+        self.assertFalse(response.context['user'].is_authenticated)
         content = response.content.decode("utf-8")
         self.assertEqual(content.find(reverse('users.user_forget')), -1)
 
@@ -1056,7 +1163,7 @@ class UserLockoutTestCase(TestCase):
 
             # should now be able to log in
             response = self.client.post(reverse('users.user_login'), post_data, follow=True)
-            self.assertTrue(response.context['user'].is_authenticated())
+            self.assertTrue(response.context['user'].is_authenticated)
 
     def testNoRecoveryNoTimeout(self):
         with self.settings(USER_ALLOW_EMAIL_RECOVERY=False, USER_LOCKOUT_TIMEOUT=-1):
@@ -1101,13 +1208,13 @@ class UserLockoutTestCase(TestCase):
 
             post_data = dict(username='plain', password='Password1')
             response = self.client.post(reverse('users.user_login'), post_data, follow=True)
-            self.assertTrue(response.context['user'].is_authenticated())
+            self.assertTrue(response.context['user'].is_authenticated)
 
 
 class PasswordExpirationTestCase(TestCase):
 
     def setUp(self):
-        self.plain = User.objects.create_user('plain', 'plain@nogroups.com', 'Password1')
+        self.plain = User.objects.create_user('plain', 'plain@nogroups.com', 'Password1 ')
         self.plain.groups.add(Group.objects.get(name="Editors"))
 
     def testNoExpiration(self):
@@ -1121,9 +1228,9 @@ class PasswordExpirationTestCase(TestCase):
 
         # log in
         self.client.logout()
-        post_data = dict(username='plain', password='Password1')
+        post_data = dict(username='plain', password='Password1 ')
         response = self.client.post(reverse('users.user_login'), post_data, follow=True)
-        self.assertTrue(response.context['user'].is_authenticated())
+        self.assertTrue(response.context['user'].is_authenticated)
 
         # we shouldn't be on a page asking us for a new password
         self.assertFalse('form' in response.context)
@@ -1133,7 +1240,7 @@ class PasswordExpirationTestCase(TestCase):
                                                  password=self.plain.password)
 
         with self.settings(USER_PASSWORD_REPEAT_WINDOW=365):
-            self.assertTrue(PasswordHistory.is_password_repeat(self.plain, "Password1"))
+            self.assertTrue(PasswordHistory.is_password_repeat(self.plain, "Password1 "))
             self.assertFalse(PasswordHistory.is_password_repeat(self.plain, "anotherpassword"))
 
             # move our history into the past
@@ -1141,19 +1248,19 @@ class PasswordExpirationTestCase(TestCase):
             history.save()
 
             # still a repeat because it is our current password
-            self.assertTrue(PasswordHistory.is_password_repeat(self.plain, "Password1"))
+            self.assertTrue(PasswordHistory.is_password_repeat(self.plain, "Password1 "))
 
             # change our password under the covers
             self.plain.set_password("my new password")
 
             # now this one is fine
-            self.assertFalse(PasswordHistory.is_password_repeat(self.plain, "Password1"))
+            self.assertFalse(PasswordHistory.is_password_repeat(self.plain, "Password1 "))
 
         with self.settings(USER_PASSWORD_REPEAT_WINDOW=-1):
             history.set_on = timezone.now()
             history.save()
 
-            self.assertFalse(PasswordHistory.is_password_repeat(self.plain, "Password1"))
+            self.assertFalse(PasswordHistory.is_password_repeat(self.plain, "Password1 "))
 
     @override_settings(USER_PASSWORD_EXPIRATION=60, USER_PASSWORD_REPEAT_WINDOW=365)
     def test_expiration(self):
@@ -1167,7 +1274,7 @@ class PasswordExpirationTestCase(TestCase):
 
         # log in
         self.client.logout()
-        post_data = dict(username='plain', password='Password1')
+        post_data = dict(username='plain', password='Password1 ')
         response = self.client.post(reverse('users.user_login'), post_data, follow=True)
 
         # assert we are being taken to our new password page
@@ -1180,16 +1287,17 @@ class PasswordExpirationTestCase(TestCase):
         # redirected again
         self.assertTrue('form' in response.context)
         self.assertTrue('new_password' in response.context['form'].fields)
+        self.assertTrue('old_password' in response.context['form'].fields)
 
         # ok, set our new password
-        post_data = dict(old_password='Password1', new_password='Password1', confirm_new_password='Password1')
+        post_data = dict(old_password='Password1 ', new_password='Password1 ', confirm_new_password='Password1 ')
         response = self.client.post(reverse('users.user_newpassword', args=[0]), post_data)
 
         # we should get a failure that our new password is a repeat
         self.assertTrue('confirm_new_password' in response.context['form'].errors)
 
         # use a different password
-        post_data = dict(old_password='Password1', new_password='Password2', confirm_new_password='Password2')
+        post_data = dict(old_password='Password1 ', new_password=' Password2', confirm_new_password=' Password2')
         response = self.client.post(reverse('users.user_newpassword', args=[0]), post_data)
 
         # should be redirected to the normal redirect page
@@ -1202,7 +1310,7 @@ class PasswordExpirationTestCase(TestCase):
         # should be able to log in normally
         self.client.logout()
 
-        post_data = dict(username='plain', password='Password2')
+        post_data = dict(username='plain', password=' Password2')
         response = self.client.post(reverse('users.user_login'), post_data)
 
         self.assertEquals(302, response.status_code)
@@ -1222,3 +1330,26 @@ class WidgetsTest(SmartminTest):
         html = widget.render('logo', img)
 
         self.assertIn('<img src="/media/2423.jpg" width="320" width="240" />', html)
+
+
+class IsPasswordComplexTest(TestCase):
+    def test_password_is_complex(self):
+        # one lower one upper one digit 8 chars
+        self.assertTrue(is_password_complex('Ab1.....'))
+        self.assertTrue(is_password_complex('   Ab1   '))
+        # password is longer than 12 chars
+        self.assertTrue(is_password_complex('longer than twelve characters'))
+
+    def test_password_is_not_complex(self):
+        # length < 8
+        self.assertFalse(is_password_complex('only5'))
+        # length < 12
+        self.assertFalse(is_password_complex('only11chars'))
+        # 8 < length < 12, no dig, no cap
+        self.assertFalse(is_password_complex('nodignocap'))
+        # 8 < length < 12, no cap
+        self.assertFalse(is_password_complex('okd1gnocap'))
+        # 8 < length < 12, no dig
+        self.assertFalse(is_password_complex('nodigokCap'))
+        # 8 < length < 12, no low
+        self.assertFalse(is_password_complex('OKD1GOKCAP'))
